from transformers import AutoProcessor, AutoModelForCausalLM
from huggingface_hub import hf_hub_download
from PIL import Image
import gradio as gr
import torch
from gradio.themes.utils import sizes


theme = gr.themes.Default(radius_size=sizes.radius_none).set(
    block_label_text_color = '#4D63FF',
    block_title_text_color = '#4D63FF',
    button_primary_text_color = '#4D63FF',
    button_primary_background_fill='#FFFFFF',
    button_primary_border_color='#4D63FF',
    button_primary_background_fill_hover='#EDEFFF',
)


processor = AutoProcessor.from_pretrained("microsoft/git-base-textvqa")
model = AutoModelForCausalLM.from_pretrained("microsoft/git-base-textvqa")


def vqa(image, question):
    inp = Image.fromarray(image.astype('uint8'), 'RGB')
    pixel_values = processor(images=inp, return_tensors="pt").pixel_values

    input_ids = processor(text=question, add_special_tokens=False).input_ids
    input_ids = [processor.tokenizer.cls_token_id] + input_ids
    input_ids = torch.tensor(input_ids).unsqueeze(0)

    generated_ids = model.generate(pixel_values=pixel_values, input_ids=input_ids, max_length=50)
    return processor.batch_decode(generated_ids, skip_special_tokens=True)


with gr.Blocks(theme=theme, css="footer {visibility: hidden}") as demo:
    gr.Markdown("""
    <div align='center' ><font size='60'>文本问答</font></div>
    """)
    with gr.Row():
        with gr.Column():
            image = gr.Image(label="图片")
            question = gr.Textbox(label="问题")
            with gr.Row():
                button = gr.Button("提交", variant="primary")
        box2 = gr.Textbox(label="文本")
    button.click(fn=vqa, inputs=[image, question], outputs=box2)
    examples = gr.Examples(examples=[['bus.png', 'what does the front of the bus say at the top?']],  inputs=[image, question], label="例子")


if __name__ == "__main__":
    demo.queue(concurrency_count=3).launch(server_name = "0.0.0.0", server_port = 7024)
